module dsp

    use real_kinds

    implicit none
    integer, parameter :: DSPERR_DOMAIN = 1

    interface theta
      module procedure theta,thetadp
    end interface
    interface rect
      module procedure rect,rectdp
    end interface
    interface sinc
      module procedure sinc,sincdp
    end interface

contains

    function theta(x) !Heaviside function, useful in DSP
        real(sp) :: theta
        real(sp), intent(in) :: x

        theta = 1.0
        if (x < 0.0 ) theta = 0.0
    end function theta

    function sinc(x) !sinc function as used in DSP
        real(sp) :: sinc, xpi
        real(sp), intent(in) :: x
        real(sp), parameter :: pi = acos(-1.0)

        xpi = x*pi
        sinc = 1.0
        if (xpi /= 0.0) sinc = sin(xpi)/xpi
    end function sinc

    function rect(t, tau, info) !generalized rectangular function, useful in DSP
        real(sp) :: rect
        real(sp), intent(in) :: t, tau
        integer, intent(out), optional :: info
        real(sp) :: abs_t, half_tau

        if (present(info)) info = 0

        if (tau <= 0.0) then
            if (present(info)) info = DSPERR_DOMAIN
            rect = 0.0
            return
        end if

        abs_t = abs(t)
        half_tau = 0.5*tau
        rect = 0.5
        if (abs_t /= half_tau) rect = theta(half_tau-abs_t)
    end function rect

    function thetadp(x) !Heaviside function, useful in DSP
        real(dp) :: thetadp
        real(dp), intent(in) :: x

        thetadp = 1.0_dp
        if (x < 0.0_dp ) thetadp = 0.0_dp
    end function thetadp

    function sincdp(x) !sinc function as used in DSP
        real(dp) :: sincdp, xpi
        real(dp), intent(in) :: x
        real(dp), parameter :: pi = acos(-1.0_dp)

        xpi = x*pi
        sincdp = 1.0_dp
        if (xpi /= 0.0_dp) sincdp = sin(xpi)/xpi
    end function sincdp

    function rectdp(t, tau, info) !generalized rectangular function, useful in DSP
        real(dp) :: rectdp
        real(dp), intent(in) :: t, tau
        integer, intent(out), optional :: info
        real(dp) :: abs_t, half_tau

        if (present(info)) info = 0

        if (tau <= 0.0_dp) then
            if (present(info)) info = DSPERR_DOMAIN
            rectdp = 0.0_dp
            return
        end if

        abs_t = abs(t)
        half_tau = 0.5_dp*tau
        rectdp = 0.5_dp
        if (abs_t /= half_tau) rectdp = theta(half_tau-abs_t)
    end function rectdp

end module dsp

